//
//  BWAppDelegate.m
//  BusinessMonitor
//
//  Created by Mark Dalrymple on 8/9/12.
//  Copyright (c) 2012 Mark Dalrymple. All rights reserved.
//

#import "BWAppDelegate.h"

#import <mach/mach_time.h>
#import "BWThingie.h"

#define SLOW 1

CGFloat BNRTimeBlock (void (^block)(void)) {
    mach_timebase_info_data_t info;
    if (mach_timebase_info(&info) != KERN_SUCCESS) return -1.0;

    uint64_t start = mach_absolute_time ();
    block ();
    uint64_t end = mach_absolute_time ();
    uint64_t elapsed = end - start;

    uint64_t nanos = elapsed * info.numer / info.denom;
    return (CGFloat)nanos / NSEC_PER_SEC;

} // BNRTimeBlock


@interface SlowCollection : NSObject

- (void) addObject: (id) object;
- (id) objectAtIndexedSubscript: (NSInteger) index;
@property (assign) NSUInteger count;

@end // SlowCollection


@interface BWAppDelegate ()
@property (nonatomic, weak) IBOutlet NSTableView *tableview;

@end // extension


@implementation BWAppDelegate {

#if SLOW
    SlowCollection *_thingies;
#else
    NSMutableArray *_thingies;
#endif
}
@synthesize tableview;


- (void) applicationDidFinishLaunching: (NSNotification *) aNotification {
    
    NSBundle *bundle = [NSBundle mainBundle];

    NSString *path = [bundle pathForResource: @"data"  ofType: @"txt"];
    
    NSString *fileContents = [NSString stringWithContentsOfFile: path
                                       encoding: NSUTF8StringEncoding
                                       error: NULL];

    NSArray *split = [fileContents componentsSeparatedByString: @"\n"];

#if SLOW
    _thingies = [SlowCollection new];
#else
    _thingies = [NSMutableArray array];
#endif

    CGFloat time = BNRTimeBlock (^{
            for (NSString *line in split) {
                if ([line hasPrefix: @"#"]) continue;
                BWThingie *thingie = [BWThingie thingieWithString: line];
                
                if (thingie == nil) continue;
                
                [_thingies addObject: thingie];
            }
        });
    NSLog (@"timing: %f", time);
    [self.tableview reloadData];
} // applicationDidFinishLaunching


- (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView {
    return _thingies.count;
} // numberOfRowsInTableView


- (id)           tableView: (NSTableView *) tableView
 objectValueForTableColumn: (NSTableColumn *) tableColumn
                       row: (NSInteger) row {
    BWThingie *thingie = _thingies[row];

    return [thingie valueForKey: tableColumn.identifier];

} // objectValueForTableColumn

@end



@interface SlowCollectionNode : NSObject

@property (strong) id object;
@property (strong) SlowCollectionNode *next;

@end // SlowCollectionNode

@implementation SlowCollectionNode
@end // SlowCollectionNode


@implementation SlowCollection {
    SlowCollectionNode *_head;
}

- (void) addObject: (id) object {
    if (_head == nil) {
        _head = [SlowCollectionNode new];
    }

    SlowCollectionNode *node = [SlowCollectionNode new];
    node.object = object;
    node.next = _head;

    node.next = _head;
    _head = node;
    _count++;
} // addObject


- (id) objectAtIndexedSubscript: (NSInteger) index {
    SlowCollectionNode *scan = _head;

    for (NSInteger i = 0; i < index; i++) {
        scan = scan.next;
    }
    return scan.object;

} // objectAtIndexedSubscript

@end // SlowCollection
